﻿using System;
using System.ComponentModel;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using Microsoft.Phone.Tasks;
using Petzold.Phone.EBookReader;

namespace BleakHouseReader
{
    public partial class MainPage : PhoneApplicationPage
    {
        AppSettings appSettings = (Application.Current as App).AppSettings;
        BookInfo book;
        WebBrowserTask webBrowserTask = new WebBrowserTask();
        SearchTask searchTask = new SearchTask();

        public MainPage()
        {
            InitializeComponent();

            // Set the fields of the appbar buttons and menu
            chaptersIconButton = this.ApplicationBar.Buttons[0] as ApplicationBarIconButton;
            slidingMenuItem = this.ApplicationBar.MenuItems[0] as ApplicationBarMenuItem;
            skewingMenuItem = this.ApplicationBar.MenuItems[1] as ApplicationBarMenuItem;
            curlingMenuItem = this.ApplicationBar.MenuItems[2] as ApplicationBarMenuItem;
            flippingMenuItem = this.ApplicationBar.MenuItems[3] as ApplicationBarMenuItem;

            switch (appSettings.PageTransition)
            {
                case "slide": OnAppBarMenuItemClick(slidingMenuItem, EventArgs.Empty); break;
                case "skew": OnAppBarMenuItemClick(skewingMenuItem, EventArgs.Empty); break;
                case "curl": OnAppBarMenuItemClick(curlingMenuItem, EventArgs.Empty); break;
                case "flip": OnAppBarMenuItemClick(flippingMenuItem, EventArgs.Empty); break;
            }

            book = appSettings.Book;

            Loaded += (object sender, RoutedEventArgs args) =>
            {
                bookViewer.JumpToPage(book.CurrentChapterIndex, book.CurrentPageIndex);
            };
        }

        protected override void OnBackKeyPress(CancelEventArgs args)
        {
            if (chaptersListBox.Visibility == Visibility.Visible)
            {
                SwitchToBookView();
                args.Cancel = true;
            }
            else if (textSelectionMenu.Visibility == Visibility.Visible)
            {
                HideTextSelectionMenu();
                args.Cancel = true;
            }

            base.OnBackKeyPress(args);
        }

        void OnChaptersListBoxSelectionChanged(object sender, SelectionChangedEventArgs args)
        {
            ChapterInfo chapter = chaptersListBox.SelectedItem as ChapterInfo;

            if (chapter != null)
            {
                book.CurrentChapterIndex = book.Chapters.IndexOf(chapter);
                book.CurrentPageIndex = 0;
                bookViewer.JumpToPage(book.CurrentChapterIndex, book.CurrentPageIndex);
            }
            SwitchToBookView();
        }

        void OnAppBarChaptersButtonClick(object sender, EventArgs args)
        {
            chaptersListBox.SelectedItem = null;

            if (chaptersListBox.Visibility == Visibility.Visible)
                SwitchToBookView();
            else
                SwitchToChaptersView();
        }

        void SwitchToBookView()
        {
            chaptersIconButton.IconUri = new Uri("Images/chapters.icon.png", UriKind.Relative);
            chaptersIconButton.Text = "chapters";
            chaptersListBox.Visibility = Visibility.Collapsed;
            bookViewer.Visibility = Visibility.Visible;
        }

        void SwitchToChaptersView()
        {
            chaptersIconButton.IconUri = new Uri("Images/text.icon.png", UriKind.Relative);
            chaptersIconButton.Text = "text";
            chaptersListBox.Visibility = Visibility.Visible;
            bookViewer.Visibility = Visibility.Collapsed;
        }

        void OnAppBarMenuItemClick(object sender, EventArgs args)
        {
            for (int i = 0; i < 4; i++)
            {
                ApplicationBarMenuItem menuItem = this.ApplicationBar.MenuItems[i] as ApplicationBarMenuItem;
                char ch = sender == menuItem ? '\x25C9' : '\x25EF';
                menuItem.Text = ch + menuItem.Text.Substring(1);
            }

            if (sender == slidingMenuItem)
            {
                appSettings.PageTransition = "slide";
                bookViewer.PageTransition = new SlideTransition();
            }

            if (sender == skewingMenuItem)
            {
                appSettings.PageTransition = "skew";
                bookViewer.PageTransition = new SkewTransition();
            }

            else if (sender == curlingMenuItem)
            {
                appSettings.PageTransition = "curl";
                bookViewer.PageTransition = new CurlTransition();
            }

            else if (sender == flippingMenuItem)
            {
                appSettings.PageTransition = "flip";
                bookViewer.PageTransition = new FlipTransition();
            }
        }

        void OnAppBarNotImplemented(object sender, EventArgs args)
        {
            MessageBox.Show("this feature is not yet implemented");
        }

        void OnBookViewerPageChanged(object sender, EventArgs args)
        {
            int chapterIndex = bookViewer.ChapterIndex;
            int pageIndex = bookViewer.PageIndex;

            if (chapterIndex < book.Chapters.Count && pageIndex < book.Chapters[chapterIndex].Pages.Count)
            {
                // Save for isolated storage settings
                book.CurrentChapterIndex = bookViewer.ChapterIndex;
                book.CurrentPageIndex = bookViewer.PageIndex;

                // Update the percentage indicator
                int accumulatedCharacterCount = book.Chapters[chapterIndex].Pages[pageIndex].AccumulatedCharacterCount;
                double percentage = 100.0 * accumulatedCharacterCount / book.CharacterCount;
                percentageRun.Text = String.Format("{0:F1}%", percentage);
            }
        }

        // When a TextSelected event comes from the BookViewer control,
        //  disable the control and display the text-selection menu.
        void OnBookViewerTextSelected(object sender, TextSelectedEventArgs args)
        {
            bookViewer.IsHitTestVisible = false;
            textSelectionMenu.Visibility = Visibility.Visible;
            textSelectionMenu.Tag = args;
            textSelectionMenu.SelectedItem = null;

            // Position the menu
            textSelectionMenu.Measure(new Size(double.PositiveInfinity, double.PositiveInfinity));
            Size selectionMenuSize = textSelectionMenu.DesiredSize;

            // Try centering it underneath the text
            if (args.Bounds.Bottom + selectionMenuSize.Height < ContentPanel.ActualHeight)
            {
                textSelectionMenuTranslate.X = (args.Bounds.Left + args.Bounds.Right - selectionMenuSize.Width) / 2;
                textSelectionMenuTranslate.Y = args.Bounds.Bottom;
            }
            // Now try centering it above the text
            else if (args.Bounds.Top - selectionMenuSize.Height > 0)
            {
                textSelectionMenuTranslate.X = (args.Bounds.Left + args.Bounds.Right - selectionMenuSize.Width) / 2;
                textSelectionMenuTranslate.Y = args.Bounds.Top - selectionMenuSize.Height;
            }
            // How about at the right?
            else if (args.Bounds.Right + selectionMenuSize.Width < ContentPanel.ActualWidth)
            {
                textSelectionMenuTranslate.X = args.Bounds.Right;
                textSelectionMenuTranslate.Y = (args.Bounds.Top + args.Bounds.Bottom - selectionMenuSize.Height) / 2;
            }
            // Maybe at the left will be OK
            else if (args.Bounds.Left - selectionMenuSize.Width > 0)
            {
                textSelectionMenuTranslate.X = args.Bounds.Left - selectionMenuSize.Width;
                textSelectionMenuTranslate.Y = (args.Bounds.Top + args.Bounds.Bottom - selectionMenuSize.Height) / 2;
            }
            // I guess we have to stick it in a corner
            else
            {
                if (args.Bounds.Left > ContentPanel.ActualWidth - args.Bounds.Right)
                    textSelectionMenuTranslate.X = 0;
                else
                    textSelectionMenuTranslate.X = ContentPanel.ActualWidth - selectionMenuSize.Width;

                if (args.Bounds.Top > ContentPanel.ActualHeight - args.Bounds.Bottom)
                    textSelectionMenuTranslate.Y = 0;
                else
                    textSelectionMenuTranslate.Y = ContentPanel.ActualHeight - selectionMenuSize.Height;
            }

            // Finally, make sure it's on the screen
            textSelectionMenuTranslate.X = Math.Max(0, Math.Min(textSelectionMenuTranslate.X, ContentPanel.ActualWidth - selectionMenuSize.Width));
            textSelectionMenuTranslate.Y = Math.Max(0, Math.Min(textSelectionMenuTranslate.Y, ContentPanel.ActualHeight - selectionMenuSize.Height));
        }

        void OnTextSelectionMenuSelectionChanged(object sender, SelectionChangedEventArgs args)
        {
            ListBox lstbox = sender as ListBox;
            TextSelectedEventArgs selectionArgs = lstbox.Tag as TextSelectedEventArgs;

            if (lstbox.SelectedItem != null)
            {
                HideTextSelectionMenu();
                string command = (lstbox.SelectedItem as ListBoxItem).Content as string;
                string text = selectionArgs.Text;

                switch (command)
                {
                    case "note":
                        MessageBox.Show("notes are not yet implemented!");
                        break;

                    case "bing":
                        searchTask.SearchQuery = StripPunctuation(text);
                        searchTask.Show();
                        break;

                    case "dictionary":
                        InvokeWebBrowser("http://www.google.com/dictionary?langpair=en|en&q={0}", text);
                        break;

                    case "wikipedia":
                        InvokeWebBrowser("http://www.wikipedia.org/wiki/{0}", text);
                        break;

                    case "exit":
                        break;
                }
            }
        }

        void HideTextSelectionMenu()
        {
            textSelectionMenu.Visibility = Visibility.Collapsed;
            bookViewer.IsHitTestVisible = true;
            bookViewer.ClearHighlighting();
        }

        void InvokeWebBrowser(string format, string text)
        {
            string url = String.Format(format, StripPunctuation(text));
            webBrowserTask.URL = Uri.EscapeDataString(url);
            webBrowserTask.Show();
        }

        string StripPunctuation(string text)
        {
            StringBuilder strBuilder = new StringBuilder();

            foreach (char ch in text)
                if (!Char.IsPunctuation(ch) || ch == '-')
                    strBuilder.Append(ch);

            return strBuilder.ToString();
        }

        void OnBookViewerBookmarkInserted(object sender, EventArgs args)
        {
            MessageBox.Show("bookmarks are not yet implemented!");
        }
    }
}